#!/usr/bin/perl -w

# $Id: NormalizeHostNames.pl,v 1.2 2005/06/08 11:42:38 root Exp $	

######################################################################
# NACE v0.4
#
# Copyright 2005 Russell Adams <rladams@adamsinfoserv.com>,
# and Adams Information Services.
#
# Licensed under GPL v2. See the README.
#
######################################################################


######################################################################
# Includes

# Use our configuration library, if not in the current directory,
# it's path had best be in the PERLLIB environment variable.
use lib "../lib";
use NACELib; 

# Other requirements
use strict;
use Getopt::Long qw( :config no_ignore_case );
use Parallel::ForkManager;
use Socket;

# End Includes
######################################################################


######################################################################
# Global vars

use vars qw( 
			 %Options
			 $ForkManager
			 );

# End Global vars
######################################################################


######################################################################
# Default Options

%Options = ( DisplayHelp     => 0,   # Do we show help and exit?
			 MaxForks        => 1,   # Effectively disable forking
			 SuffixRegExp    => 0,   # Domain suffix to remove
			 );

# End of default options
######################################################################


######################################################################
# Main Program
######################################################################

ProcessOptions();

DebugMsg('Beginning hostname normalization...');

# Setup forking
DebugMsg("Using Parallel ForkManager, maximum $Options{'MaxForks'} children.");

$ForkManager = new Parallel::ForkManager( $Options{'MaxForks'} );

# read host list from STDIN
DebugMsg('Reading hosts from STDIN.');

while ( <STDIN> ) {

	chomp();

	# engage a new fork.
	my $pid = $ForkManager->start and next;

	# split out hostname
	my $InputHostName = ( split(/:/, $_) )[0];

	# hash out any additional host data
	my $InputHostData = CreateHashFromString( '[A-Z][A-Z]', $_ );
	
	NormalizeHost( $InputHostName, $InputHostData );
	
	# End fork
	$ForkManager->finish;
	
};

# Wait til all forks complete
$ForkManager->wait_all_children;

######################################################################
# End Main Program
######################################################################


######################################################################
# Functions
######################################################################


######################################################################
# ProcessOptions

sub ProcessOptions {

	GetOptions( 
				'D'         => \$Debug,
				'h'         => \$Options{'DisplayHelp'},
				'f=i'       => \$Options{'MaxForks'},
				's=s'       => \$Options{"SuffixRegExp"},
				);

	$Debug && DebugDumpHash( 'Options', \%Options );

	$Options{'DisplayHelp'} && DisplayHelp();

    ########################################
    # Check for required options

	$Options{'SuffixRegExp'} || die "A suffix regexp must be specified (-s).\n";


	DebugMsg('Options processing complete.');

};

# End ProcessOptions
######################################################################


######################################################################
# NormalizeHost

sub NormalizeHost {

	my ( $HostOrig, $HostData ) = @_ ;

	DebugMsg("Normalizing host: $HostOrig");
	DebugDumpHash("Host data", $HostData);

	my $Packed = inet_aton( $HostOrig );

	# The original host name is the default for all fields

	my $HostIP = my $HostName = my $HostFullName = $HostOrig;

	# Did we get a packed ip, or null?

	if ( $Packed ) {
		
		$HostIP = inet_ntoa( $Packed );

		$HostIP || DebugMsg('Bad Host ip returned.');

		$HostFullName = gethostbyaddr( $Packed , AF_INET);

		if ( ! $HostFullName ) { $HostFullName = $HostIP; };

		# If the name returned doesn't match ours, then don't fix it.
		# The IP resolves, so thats all we ask.
		if ( $HostFullName !~ /$HostOrig/ ) { 
		    $HostFullName = $HostOrig;
		};

		# chop off the suffix.

		DebugMsg("Full name: $HostFullName");
		$HostName = $HostFullName;
		my $RegExp = $Options{'SuffixRegExp'};
		$HostName =~ s/$RegExp//g;
		
	};

	$HostData->{'HN'} = $HostName;
	$HostData->{'AD'} = $HostIP;

	my $OutputRecord = CreateStringFromHash( $HostData );

	print $HostName . $OutputRecord . "\n";

};

# End SNMPScanHost
######################################################################


######################################################################
# DisplayHelp

sub DisplayHelp {

	print <<EOF;

NormalizeHostNames.pl

Reads host names and host data as a colon delimited record on STDIN,
and performs DNS lookups against hosts as required. Replaces host with
domain name minus the suffix regexp, and assigns the host_name and
address fields with the domain name and the FIRST matching IP address.

Syntax:

	-s <regexp>

		Regexp which is used to remove domain suffix.

	-f <forks>

		Maximum number of forks to spawn to scan hosts, defaults to 1.

	-D 

		Debug, enables very verbose debugging information.

EOF

	exit -1;

};

# End DisplayHelp
######################################################################
