#!/usr/bin/perl -w

# $Id: SNMPHostGrep.pl,v 1.6 2005/06/09 00:39:57 root Exp $	

######################################################################
# NACE v0.4
#
# Copyright 2005 Russell Adams <rladams@adamsinfoserv.com>,
# and Adams Information Services.
#
# Licensed under GPL v2. See the README.
#
######################################################################


######################################################################
# Includes

# Use our configuration library, if not in the current directory,
# it's path had best be in the PERLLIB environment variable.
use lib "../lib";
use NACELib; 

# Other requirements
use strict;
use Net::SNMP; # Tested on v3.6, on Debian Woody.
use Getopt::Long qw( :config no_ignore_case );
use Parallel::ForkManager;

# End Includes
######################################################################


######################################################################
# Global vars

use vars qw( 
			 %Options
			 $ForkManager
			 );

# End Global vars
######################################################################


######################################################################
# Default Options

%Options = ( DisplayHelp     => 0,   # Do we show help and exit?
			 Community       => 'public', # Default community
			 Port            => '161',    # Default port
			 MaxForks        => 1,   # Effectively disable forking
			 TargetOID       => '1.3.6.1.2.1.1.1.0',
                                     # OID to query, default sysDescr
			 TargetRegexp    => 0,   # Regexp to match OID to
			 );

# End of default options
######################################################################


######################################################################
# Main Program
######################################################################

ProcessOptions();

DebugMsg('Beginning SNMP scanning...');

# Setup forking
DebugMsg("Using Parallel ForkManager, maximum $Options{'MaxForks'} children.");

$ForkManager = new Parallel::ForkManager( $Options{'MaxForks'} );

# read host list from STDIN
DebugMsg('Reading hosts from STDIN.');

while ( <STDIN> ) {

	chomp();

	# engage a new fork.
	my $pid = $ForkManager->start and next;
	
	# split out hostname
	my $InputHostName = ( split(/:/, $_) )[0];

	# hash out any additional host data
	my $InputHostData = CreateHashFromString( '[A-Z][A-Z]', $_ );
	
	SNMPScanHost( $InputHostName, $InputHostData );
	
	# End fork
	$ForkManager->finish;
	
};

# Wait til all forks complete
$ForkManager->wait_all_children;

######################################################################
# End Main Program
######################################################################


######################################################################
# Functions
######################################################################


######################################################################
# ProcessOptions

sub ProcessOptions {

	GetOptions( 
				'D'         => \$Debug,
				'h'         => \$Options{'DisplayHelp'},
				'c=s'       => \$Options{'Community'},
				'p=i'       => \$Options{'Port'},
				'f=i'       => \$Options{'MaxForks'},
				'o=s'       => \$Options{"TargetOID"},
				'm=s'       => \$Options{"TargetRegexp"},
				);

	$Debug && DebugDumpHash( 'Options', \%Options );

	$Options{'DisplayHelp'} && DisplayHelp();

    ########################################
    # Check for required options

	$Options{'TargetRegexp'} || die "A regexp must be specified (-m).\n";


	DebugMsg('Options processing complete.');

};

# End ProcessOptions
######################################################################


######################################################################
# SNMPScanHost

sub SNMPScanHost {

	my ( $HostName, $HostData ) = @_ ;

	DebugMsg("Scanning $_");

	my ( $SNMPSession , $SNMPError, $SNMPResponse, $RemoteValue );

	# Create session object (does NOT open a connection, its stateless)
	# What about version, timeout, retries, debug mode?
	( $SNMPSession , $SNMPError ) = Net::SNMP->session( 
									   -hostname    => $HostName,
									   -community   => $Options{'Community'},
									   -port        => $Options{'Port'},
									   );

	# Check session object was created
	if ( ! defined $SNMPSession ) {
		print STDERR "$SNMPError\n";
		return undef;
	};
		
	# Query sysDescr
	$SNMPResponse = $SNMPSession->get_request( $Options{'TargetOID'} );

	# If not defined, there was no response.
	if ( ! defined $SNMPResponse ) {

		DebugMsg("SNMPScanHost $HostName failed: " . $SNMPSession->error());
		return undef;

	};
		
	############################################
	# SNMP responded, continue to evaluate host.
	$RemoteValue = $SNMPResponse->{ $Options{'TargetOID'} };
	
	DebugMsg("Retrived $Options{'TargetOID'} from $HostName: $RemoteValue");
	
	# Finished with SNMP.
	$SNMPSession->close();

	################################################
	# Test regexp vs value
	if ( $RemoteValue !~ /$Options{'TargetRegexp'}/ ) {
			
			# Host sysDescr didn't match inclusion regexp, skip
			DebugMsg("$HostName did not match /$Options{'TargetRegExp'}/, skipping.");
			return undef;

			};

	##################################################
    # Host matched, so output!
	DebugMsg("$HostName matched, including.");
	
	# Prep output record and print
	my $OutputRecord = CreateStringFromHash( $HostData );

	print $HostName . $OutputRecord . "\n";

};

# End SNMPScanHost
######################################################################


######################################################################
# DisplayHelp

sub DisplayHelp {

	print <<EOF;

SNMPHostGrep.pl

Reads hostnames or IP addresses via STDIN. Performs an SNMP get on
each host for sysDescr or the OID specified, and matches it against
the regexp. If the value returned by the host matched the regexp, the
hostname given is printed to STDOUT. Hosts that fail to match are
omitted.

Syntax:

	-m <regexp>

		Regexp used to match against values retrieved from hosts.

	-o <OID>

		OID to retrieve from each host, defaults to sysDescr.

	-c <community>

		SNMP community to use when polling, defaults to public.

	-p <port>

		SNMP port to use when polling, defaults to 169.

	-f <forks>

		Maximum number of forks to spawn to scan hosts, defaults to 1.

	-D 

		Debug, enables very verbose debugging information.

EOF

	exit -1;

};

# End DisplayHelp
######################################################################
