#!/usr/bin/perl -w

# $Id: SNMPServiceOIDGrep.pl,v 1.6 2005/06/10 10:22:23 root Exp $	

######################################################################
# NACE v0.4
#
# Copyright 2005 Russell Adams <rladams@adamsinfoserv.com>,
# and Adams Information Services.
#
# Licensed under GPL v2. See the README.
#
######################################################################


######################################################################
# Includes

# Use our configuration library, if not in the current directory,
# it's path had best be in the PERLLIB environment variable.
use lib "../lib";
use NACELib; 

# Other requirements
use strict;
use Net::SNMP; # Tested on v3.6, on Debian Woody.
use Getopt::Long qw( :config no_ignore_case );
use Parallel::ForkManager;

# End Includes
######################################################################


######################################################################
# Global vars

use vars qw( 
			 %Options
			 $ForkManager
			 );

# End Global vars
######################################################################


######################################################################
# Default Options

%Options = ( DisplayHelp     => 0,   # Do we show help and exit?
			 Community       => 'public', # Default community
			 Port            => '161',    # Default port
			 MaxForks        => 1,   # Effectively disable forking
			 TargetOID       => 0,   # OID to query
			 TargetRegExp    => 0,   # Regexp to match OID to
			 TargetExp       => 0,   # Target expression
	                 TargetOIDDesc   => 0,   # Get desc from ID
			 TargetDesc      => '@@@TARGET@@@' # Description
			 );

# End of default options
######################################################################


######################################################################
# Main Program
######################################################################

ProcessOptions();

DebugMsg('Beginning SNMP scanning...');

# Setup forking
DebugMsg("Using Parallel ForkManager, maximum $Options{'MaxForks'} children.");

$ForkManager = new Parallel::ForkManager( $Options{'MaxForks'} );

# read host list from STDIN
DebugMsg('Reading hosts from STDIN.');

while ( <STDIN> ) {

	chomp();

	# engage a new fork.
	my $pid = $ForkManager->start and next;
	
	# split out hostname
	my $InputHostName = ( split(/:/, $_) )[0];

	# hash out any additional host data
	my $InputHostData = CreateHashFromString( '[A-Z][A-Z]', $_ );
	
	SNMPServiceScanHost( $InputHostName, $InputHostData );
	
	# End fork
	$ForkManager->finish;
	
};

# Wait til all forks complete
$ForkManager->wait_all_children;

######################################################################
# End Main Program
######################################################################


######################################################################
# Functions
######################################################################


######################################################################
# ProcessOptions

sub ProcessOptions {

	GetOptions( 
				'D'         => \$Debug,
				'h'         => \$Options{'DisplayHelp'},
				'c=s'       => \$Options{'Community'},
				'p=i'       => \$Options{'Port'},
				'f=i'       => \$Options{'MaxForks'},
				'o=s'       => \$Options{"TargetOID"},
				'm=s'       => \$Options{"TargetRegExp"},
				's=s'       => \$Options{'TargetExp'},
				'd=s'       => \$Options{"TargetDesc"},
				't=s'       => \$Options{"TargetOIDDesc"},
				);

	$Debug && DebugDumpHash( 'Options', \%Options );

	$Options{'DisplayHelp'} && DisplayHelp();

    ########################################
    # Check for required options

	$Options{'TargetOID'}    || die "An OID must be specified (-o).\n";
	$Options{'TargetRegExp'} || die "A regexp must be specified (-m).\n";
	$Options{'TargetExp'}    || die "A service expression  must be specified (-s).\n";


	DebugMsg('Options processing complete.');

};

# End ProcessOptions
######################################################################


######################################################################
# SNMPServiceScanHost

sub SNMPServiceScanHost {

	my ( $HostName, $HostData ) = @_ ;

	DebugMsg("Scanning $HostName");

	my ( $SNMPSession , $SNMPError, $SNMPResponse,
	     $RemoteValue, $LookupResponse );

	# Create session object (does NOT open a connection, its stateless)
	# What about version, timeout, retries, debug mode?
	( $SNMPSession , $SNMPError ) = Net::SNMP->session( 
									   -hostname    => $HostName,
									   -community   => $Options{'Community'},
									   -port        => $Options{'Port'},
									   );

	# Check session object was created
	if ( ! defined $SNMPSession ) {
		print STDERR "$SNMPError\n";
		return undef;
	};

	#######################################
	# Fetch table matching TargetOID
	$SNMPResponse = $SNMPSession->get_table( $Options{'TargetOID'} );
	
	# If not defined, there was no response.
	if ( ! defined $SNMPResponse ) {

		DebugMsg("SNMPScanHost $HostName failed: " . $SNMPSession->error());
		return undef;

	};
		
	DebugMsg "Retrived SNMP table $Options{'TargetOID'} to scan." ;
	DebugDumpHash ( 'Table' , $SNMPResponse );

	#######################################
	# Try to match each value returned.
	foreach ( sort keys %$SNMPResponse ) {

		DebugMsg "Testing $SNMPResponse->{$_} against /$Options{'TargetRegExp'}/";

		$SNMPResponse->{ $_ } =~ /$Options{'TargetRegExp'}/ || next;
			
		DebugMsg "Adding target for $SNMPResponse->{$_}.";

		# Sanitize the value first of bad metachars
		my $SanitizedName = $SNMPResponse->{ $_ };

		my $LastIndex = ( split(/\./) )[-1];

		# Optional lookup for description here.
		$Options{'TargetOIDDesc'} && do {

		    DebugMsg "Performing lookup for target desc for index $LastIndex.";

		    $LookupResponse = $SNMPSession->get_request( $Options{'TargetOIDDesc'} . '.' . $LastIndex );

		    # If not defined, there was no response.
		    if ( ! defined $LookupResponse ) {
			
			DebugMsg("OID Lookup $HostName failed: " . $SNMPSession->error());
			return undef;
			
		    };
		    
		    # Override target desc with value from remote oid
		    $SanitizedName =
			$LookupResponse->{ $Options{'TargetOIDDesc'} . '.' . $LastIndex};

		};

		# Now sanitize
		$SanitizedName =~ tr/\\!\#$`:=/./;

		# create output
		$HostData->{'SC'} = $Options{'TargetExp'};
		$HostData->{'SD'} = $Options{'TargetDesc'};

		$HostData = PerformSubst( $HostData, '\@\@\@TARGET\@\@\@',
								  $SanitizedName );

		my $OutputRecord = CreateStringFromHash( $HostData );

		# print results
		print $HostName . $OutputRecord . "\n";

		};

	# Finished with SNMP.
	$SNMPSession->close();
};

# End SNMPServiceScanHost
######################################################################


######################################################################
# DisplayHelp

sub DisplayHelp {

	print <<EOF;

SNMPServiceOIDGrep.pl

Reads hostnames or IP addresses via STDIN. Performs an SNMP get table
on each host for the OID specified, and matches it against the
regexp. If the value returned by the host matched the regexp, the
hostname and service check expression are printed to STDOUT. Hosts
that fail to match are omitted.

Syntax:

	-s <expression>

		check_command string, where '\@\@\@TARGET\@\@\@' 
		is replaced by the hostname.

        -d <expression>

                service_description string, with replacement.

	-m <regexp>

		Regexp used to match against values retrieved from hosts.

        -t <OID>
                Fetch description from this OID prefix for matching rows.

	-o <OID>

		OID to retrieve from each host, defaults to sysDescr.

	-c <community>

		SNMP community to use when polling, defaults to public.

	-p <port>

		SNMP port to use when polling, defaults to 169.

	-f <forks>

		Maximum number of forks to spawn to scan hosts, defaults to 1.

	-D 

		Debug, enables very verbose debugging information.

EOF

	exit -1;

};

# End DisplayHelp
######################################################################
