#!/usr/bin/perl -w

# $Id: SNMPServiceQueryKey.pl,v 1.4 2005/06/10 10:07:07 root Exp $	

######################################################################
# NACE v0.4
#
# Copyright 2005 Russell Adams <rladams@adamsinfoserv.com>,
# and Adams Information Services.
#
# Licensed under GPL v2. See the README.
#
######################################################################


######################################################################
# Includes

# Use our configuration library, if not in the current directory,
# it's path had best be in the PERLLIB environment variable.
use lib "../lib";
use NACELib; 

# Other requirements
use strict;
use Net::SNMP; # Tested on v3.6, on Debian Woody.
use Getopt::Long qw( :config no_ignore_case );
use Parallel::ForkManager;

# End Includes
######################################################################


######################################################################
# Global vars

use vars qw( 
			 %Options
			 $ForkManager
			 );

# End Global vars
######################################################################


######################################################################
# Default Options

%Options = ( DisplayHelp     => 0,        # Do we show help and exit?
			 Community       => 'public', # Default community
			 Port            => '161',    # Default port
			 MaxForks        => 1,        # Effectively disable forking
			 TargetKey       => 0,        # OID to query
			 TargetDesc      => '@@@TARGET@@@' # Description
			 );

# End of default options
######################################################################


######################################################################
# Main Program
######################################################################

ProcessOptions();

DebugMsg('Beginning SNMP scanning...');

# Setup forking
DebugMsg("Using Parallel ForkManager, maximum $Options{'MaxForks'} children.");

$ForkManager = new Parallel::ForkManager( $Options{'MaxForks'} );

# read host list from STDIN
DebugMsg('Reading hosts from STDIN.');

while ( <STDIN> ) {

	# engage a new fork.
	my $pid = $ForkManager->start and next;

	chomp();

	# split out hostname
	my $InputHostName = ( split(/:/, $_) )[0];

	# hash out any additional host data
	my $InputHostData = CreateHashFromString( '[A-Z][A-Z]', $_ );
	
	SNMPServiceQueryHost( $InputHostName, $InputHostData );
	
	# End fork
	$ForkManager->finish;
	
};

# Wait til all forks complete
$ForkManager->wait_all_children;

######################################################################
# End Main Program
######################################################################


######################################################################
# Functions
######################################################################


######################################################################
# ProcessOptions

sub ProcessOptions {

	GetOptions( 
				'D'         => \$Debug,
				'h'         => \$Options{'DisplayHelp'},
				'c=s'       => \$Options{'Community'},
				'p=i'       => \$Options{'Port'},
				'f=i'       => \$Options{'MaxForks'},
				's=s'       => \$Options{"TargetExp"},
				'd=s'       => \$Options{"TargetDesc"},
				'k=s'       => \$Options{"TargetKey"},
				);

	$Debug && DebugDumpHash( 'Options', \%Options );

	$Options{'DisplayHelp'} && DisplayHelp();

    ########################################
    # Check for required options

	$Options{'TargetKey'}    || die "A key must be specified (-k).\n";
	$Options{'TargetExp'}    || die "A service expression  must be specified (-s).\n";


	DebugMsg('Options processing complete.');

};

# End ProcessOptions
######################################################################


######################################################################
# SNMPServiceScanHost

sub SNMPServiceQueryHost {

	my ( $HostName, $HostData ) = @_ ;

	DebugMsg("Scanning $HostName");

	my ( $SNMPSession , $SNMPError, $SNMPResponse, $RemoteData);

	my $RemoteValue = '';

	# Create session object (does NOT open a connection, its stateless)
	# What about version, timeout, retries, debug mode?
	( $SNMPSession , $SNMPError ) = Net::SNMP->session( 
									   -hostname    => $HostName,
									   -community   => $Options{'Community'},
									   -port        => $Options{'Port'},
									   );

	# Check session object was created
	if ( ! defined $SNMPSession ) {
		print STDERR "$SNMPError\n";
		return undef;
	};

	########################################
	# Query given remote tags
	$RemoteData = GetConfigurationDirectives( $SNMPSession , '..' );

	DebugDumpHash( "RemoteData for $HostName" , $RemoteData );

	# if there was data for the key returned, use each value
	if ( $RemoteData->{ $Options{'TargetKey'} } ) {

		$RemoteValue = $RemoteData->{ $Options{'TargetKey'} };

	};

	foreach ( split(/,/ , $RemoteValue) ) {

		DebugMsg "Adding target for $HostName.";

		# Sanitize the value first of bad metachars
		my $SanitizedName = $HostName;
		$SanitizedName =~ tr/\\!\#$`:=/./;

		# create output
		$HostData->{'SC'} = $Options{'TargetExp'};
		$HostData->{'SD'} = $Options{'TargetDesc'};

		$HostData = PerformSubst( $HostData, '\@\@\@TARGET\@\@\@',
								  $SanitizedName );

		my $OutputRecord = CreateStringFromHash( $HostData );

		# print results
		print $HostName . $OutputRecord . "\n";

		};

	# Finished with SNMP.
	$SNMPSession->close();

};

# End SNMPServiceScanHost
######################################################################


######################################################################
# DisplayHelp

sub DisplayHelp {

	print <<EOF;

SNMPServiceQueryKey.pl

Reads hostnames or IP addresses via STDIN. Fetches a NACE
configuration directive by its key on each host via SNMP. Prints one
service check expression for each item in the comma delimited list in
the key\'s value.

Syntax:

	-s <expression>

		check_command string, where '\@\@\@TARGET\@\@\@' 
		is replaced by the target string.

    -d <expression>

        service_description string, with replacement.

	-k <key>

		NACE configuration key to retrieve from each host.

	-c <community>

		SNMP community to use when polling, defaults to public.

	-p <port>

		SNMP port to use when polling, defaults to 169.

	-f <forks>

		Maximum number of forks to spawn to scan hosts, defaults to 1.

	-D 

		Debug, enables very verbose debugging information.

EOF

	exit -1;

};

# End DisplayHelp
######################################################################
